package com.android.launcher3.accessibility;

import android.accessibilityservice.AccessibilityService;
import android.accessibilityservice.AccessibilityServiceInfo;
import android.app.ActivityManager;
import android.content.ComponentName;
import android.content.Intent;
import android.content.pm.ActivityInfo;
import android.content.pm.PackageManager;
import android.text.TextUtils;
import android.view.KeyEvent;
import android.view.accessibility.AccessibilityEvent;
import android.widget.Toast;

import com.android.launcher3.Launcher;
import com.android.launcher3.SettingsActivity;
import com.android.launcher3.manager.ColumnDef;
import com.android.launcher3.manager.Constants;
import com.android.launcher3.manager.LauncherManager;
import com.android.launcher3.manager.ManagementInfo;
import com.android.launcher3.manager.ManagerAppProvider;
import com.android.launcher3.util.Logger;

/**
 * 拦截app的启动
 */
public class AppManagerAccessiblityService extends AccessibilityService {

    public static final String TAG = "AppManagerAccessiblityService";


    @Override
    protected void onServiceConnected() {
        super.onServiceConnected();
        Logger.e(TAG, "onServiceConnected\t");
    }

    @Override
    public void onAccessibilityEvent(AccessibilityEvent event) {
        int eventType = event.getEventType();
        if (eventType == AccessibilityEvent.TYPE_WINDOW_STATE_CHANGED) {
            String packageName = event.getPackageName().toString();

            if (TextUtils.isEmpty(packageName)) {
                packageName = "";
            }
            if (packageName.contains("com.android.settings")) {
                backHome(-1);
                LauncherManager.getInstance().jumpSetting(this);
            }

            ManagementInfo appinfo = LauncherManager.getInstance().getManageAppInfo(packageName);
            if (appinfo != null) {
                if (appinfo.getLock() == ColumnDef.LockApp.LOCKED) {
                    backHome(Constants.DIALOG_TYPE_LOCKED);
                }
                if (appinfo.getLock() == ColumnDef.LockApp.DEADLINE) {
                    backHome(Constants.DIALOG_TYPE_APP_USE_DEADLINE);
                }

                if (appinfo.getUseType() == ColumnDef.AppUseType.UNCLASS_TYPE) {
                    backHome(Constants.DIALOG_TYPE_APP_UNCLASS);
                }
            }
            Logger.e(TAG, event.toString());
        } else if (eventType == AccessibilityEvent.TYPE_VIEW_CLICKED) {

            Logger.e(TAG, event.toString());
        }

    }

    private void backHome(int type) {
        LauncherManager.getInstance().backLauncherHome(getApplicationContext(), type);
    }


    @Override
    public void onInterrupt() {
        Logger.e(TAG, "onInterrupt");
    }


}
