package com.android.launcher3.accessibility;

import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.app.Service;
import android.app.usage.UsageStats;
import android.app.usage.UsageStatsManager;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.graphics.Color;
import android.os.Build;
import android.os.Handler;
import android.os.IBinder;
import android.os.Message;

import com.android.launcher3.Launcher;
import com.android.launcher3.R;
import com.android.launcher3.config.AppConfig;
import com.android.launcher3.manager.ColumnDef;
import com.android.launcher3.manager.Constants;
import com.android.launcher3.manager.LauncherManager;
import com.android.launcher3.manager.ManagementInfo;
import com.android.launcher3.model.ConfigManager;
import com.android.launcher3.util.Logger;
import com.android.launcher3.util.NotificationUtils;

import java.util.Arrays;
import java.util.List;

import androidx.annotation.Nullable;
import androidx.core.app.NotificationCompat;

/**
 * 后台监听app的活动
 */
public class ListenerAppService extends Service {

    public static final String TAG = "ListenerAppService";

    private static final int TIME = 60 * 1000;

    private static final int LOOP_TIME = 100;

    private boolean active = true;

    private InnerRecevier innerReceiver;


    private Handler handler = new Handler() {
        @Override
        public void handleMessage(Message msg) {
            if (active) {
                post(checkAppRunnable);
            }
            handler.sendEmptyMessageDelayed(0, LOOP_TIME);
        }
    };

    private CheckAppRunnable checkAppRunnable;

//    private

    @Nullable
    @Override
    public IBinder onBind(Intent intent) {
        return null;
    }


    @Override
    public void onCreate() {
        super.onCreate();
        //创建广播
        innerReceiver = new InnerRecevier();
        //动态注册广播
        IntentFilter intentFilter = new IntentFilter(Intent.ACTION_CLOSE_SYSTEM_DIALOGS);
        intentFilter.addAction(Intent.ACTION_SCREEN_OFF);
        intentFilter.addAction(Intent.ACTION_SCREEN_ON);
//启动广播
        registerReceiver(innerReceiver, intentFilter);


//        registerReceiver()

        Logger.e(TAG, "onCreate");

        checkAppRunnable = new CheckAppRunnable();

    }

    @Override
    public int onStartCommand(Intent intent, int flags, int startId) {

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            NotificationUtils.ChannelBuilder channelBuilder = new NotificationUtils.ChannelBuilder("前台服务", "1111", "锁屏保护", NotificationManager.IMPORTANCE_DEFAULT)
                    .setChannelName("锁屏保护")
                    .setByPassDnd(false) //是否绕过免打扰
                    .setLightColor(Color.GREEN) //LED指示灯的颜色
                    .setShowBadge(false) //是否在桌面显示未读消息数量
                    .setEnableLight(false) //是否需要显示指示灯
                    .setEnableSound(false) //是否开启铃声
                    .setEnableVibrate(false) //是否开启震动
                    .setVisibility(NotificationCompat.VISIBILITY_SECRET); //锁屏界面是否可见


            NotificationUtils notificationUtils = new NotificationUtils(getApplicationContext(), channelBuilder);

            NotificationCompat.Builder builder = new NotificationCompat.Builder(getApplicationContext(), notificationUtils.getCurrentChannel());
            startForeground(1, builder.build());

        }


        handler.sendEmptyMessageDelayed(0, LOOP_TIME);

//        usageStatsManager.
        return START_STICKY;
    }


    @Override
    public void onDestroy() {
        if (innerReceiver != null) {
            unregisterReceiver(innerReceiver);
        }
        super.onDestroy();
    }

    public class InnerRecevier extends BroadcastReceiver {

        final String SYSTEM_DIALOG_REASON_KEY = "reason";

        final String SYSTEM_DIALOG_REASON_RECENT_APPS = "recentapps";

        final String SYSTEM_DIALOG_REASON_HOME_KEY = "homekey";

        @Override
        public void onReceive(Context context, Intent intent) {

            String action = intent.getAction();

            Logger.e(TAG, action);

            if (Intent.ACTION_CLOSE_SYSTEM_DIALOGS.equals(action)) {
                String reason = intent.getStringExtra(SYSTEM_DIALOG_REASON_KEY);
                if (reason != null) {
                    Logger.e(TAG, action + "\t" + reason);
                    if (reason.equals(SYSTEM_DIALOG_REASON_HOME_KEY)) {
                        // "Home键被监听"
                        if (!LauncherManager.getInstance().isForceLock()) {
                            LauncherManager.getInstance().backLauncherHome(getApplicationContext());
                        }
                    } else if (reason.equals(SYSTEM_DIALOG_REASON_RECENT_APPS)) {
                        if (!LauncherManager.getInstance().isForceLock()) {
                            LauncherManager.getInstance().backLauncherHome(getApplicationContext());
                        }
                    }
                }
            } else if (Intent.ACTION_SCREEN_OFF.equals(action)) {
                active = false;
            } else if (Intent.ACTION_SCREEN_ON.equals(action)) {
                active = true;
            }
        }
    }


    public class CheckAppRunnable implements Runnable {

        @Override
        public void run() {
            if (ConfigManager.getInstance().isLockScreen()) {
                return;
            }
            UsageStatsManager usageStatsManager = (UsageStatsManager) getSystemService(USAGE_STATS_SERVICE);

            long currentMills = System.currentTimeMillis();

            List<UsageStats> stats = usageStatsManager.queryUsageStats(UsageStatsManager.INTERVAL_BEST, currentMills - TIME, currentMills);

            //取得当前在前台的app，即当前运行的app
            if ((stats != null) && (!stats.isEmpty())) {
                String topActivity = "";
                int j = 0;
                for (int i = 0; i < stats.size(); i++) {
                    if (stats.get(i).getLastTimeUsed() > stats.get(j).getLastTimeUsed()) {
                        j = i;
                    }
                }
                topActivity = stats.get(j).getPackageName();

                Logger.e(TAG, topActivity);

                if (topActivity.equals(getPackageName()))
                    return;
                if (LauncherManager.getInstance().getLockScreenMode() != ColumnDef.LockScreen.MODE_UNLOCK) {

                    if (LauncherManager.getInstance().isForceLock() && !LauncherManager.getInstance().isSafePackageName(topActivity)) {

                        if (!ConfigManager.getInstance().isLockScreen()) {
                            LauncherManager.getInstance().lockScreen(getApplicationContext(), LauncherManager.getInstance().getModeDesc1(), LauncherManager.getInstance().getModeDesc2(), LauncherManager.getInstance().getLockScreenMode());
                        }
                    }
                }


                if ("com.android.settings".equals(topActivity)) {
                    LauncherManager.getInstance().backLauncherHome(getApplicationContext());
                    LauncherManager.getInstance().jumpSetting(getApplicationContext());
                }


                ManagementInfo appinfo = LauncherManager.getInstance().getManageAppInfo(topActivity);
                if (appinfo != null) {
                    if (appinfo != null) {
                        if (appinfo.getLock() == ColumnDef.LockApp.LOCKED) {

                            LauncherManager.getInstance().backLauncherHome(getApplicationContext(), Constants.DIALOG_TYPE_LOCKED);
                        }
                        if (appinfo.getLock() == ColumnDef.LockApp.DEADLINE) {
                            LauncherManager.getInstance().backLauncherHome(getApplicationContext(), Constants.DIALOG_TYPE_APP_USE_DEADLINE);

                        }

                        if (appinfo.getUseType() == ColumnDef.AppUseType.UNCLASS_TYPE) {
                            LauncherManager.getInstance().backLauncherHome(getApplicationContext(), Constants.DIALOG_TYPE_APP_UNCLASS);
                        }
                    }
                }
            }

        }
    }
}


